# This file was auto-generated by Fern from our API Definition.

import typing
from ...types.background_manager import BackgroundManager
from ...types.dynamic_manager import DynamicManager
from ...types.round_robin_manager import RoundRobinManager
from ...types.supervisor_manager import SupervisorManager

GroupCreateManagerConfig = typing.Union[BackgroundManager, DynamicManager, RoundRobinManager, SupervisorManager]
